# encoding: utf-8
require 'json'
require "#{File.expand_path(File.dirname(__FILE__))}/../spec_helper"
require 'logstash/inputs/cloudflare_logs'
require 'pry'

describe CloudflareAccess do
  let(:meta_filename) { "#{File.expand_path(File.dirname(__FILE__))}/../tmp/metadata.json" }
  let(:logger) { logger = double() }

  let(:cloudflare_access) do
    CloudflareAccess.new(auth_email: 'test@test.com',
                         auth_key: 'secret',
                         domain: '11',
                         logger: logger,
                         metadata_file: meta_filename)
  end

  describe '#start_time' do
    subject { cloudflare_access.start_time }
    it 'should return current time - 15 min if metadata file is not there' do
      `rm #{meta_filename}`
      Timecop.freeze do
        expect(subject).to eq((Time.now - (15 * 60)).to_datetime.rfc3339)
      end
    end

    it 'should return the default time meta file is there but is empty' do
      `rm #{meta_filename}`
      `touch #{meta_filename}`

      Timecop.freeze do
        expect(subject).to eq((Time.now - (15 * 60)).to_datetime.rfc3339)
      end
    end

    it 'should return the start time in the file if it is correct' do
      Timecop.freeze do
        File.open(meta_filename, 'w+') { |file| file.write({ start_time: Time.now - (24 * 60 * 60) }.to_json) }
        expect(subject).to eq((Time.now - (24 * 60 * 60)).to_datetime.rfc3339)
      end
    end
  end

  describe '#end_time' do
    subject { cloudflare_access.end_time }

    it 'should return 5 min before current time' do
      Timecop.freeze do
        expect(subject).to eq((Time.now - (5 * 60)).to_datetime.rfc3339)
      end
    end
  end

  describe '#update_metadata_file' do
    def read_meta_data
      Time.parse(JSON.parse(File.read(meta_filename))['start_time']).to_datetime.rfc3339
    end

    before(:all) { Timecop.freeze }
    after(:all) { Timecop.freeze }
    subject { cloudflare_access.update_metadata_file(:start_time, Time.now) }

    it 'should set time if the metadata file does not exist' do
      `rm #{meta_filename}`

      subject

      expect(read_meta_data).to eq Time.now.to_datetime.rfc3339
    end

    it 'should set the value of the meta data if the file exists' do
      subject
      @value = Time.now - (14 * 60)
      cloudflare_access.update_metadata_file(:start_time, @value)

      expect(read_meta_data).to eq((Time.now - (14 * 60)).to_datetime.rfc3339)
    end

    it 'should work correctly if the meta data file is there and blank' do
      `rm #{meta_filename}`
      `touch #{meta_filename}`
      subject

      expect(read_meta_data).to eq Time.now.to_datetime.rfc3339
    end
  end

  describe '#logs' do
    before :each do
      allow(cloudflare_access).to receive(:start_time).and_return('2017-10-10T23:54:57Z')
      allow(cloudflare_access).to receive(:end_time).and_return('2017-10-10T23:55:00Z')
    end

    subject do
      VCR.use_cassette(:logs) do
        cloudflare_access.logs
      end
    end

    it 'it should return the 4 log hashes' do
      expect(subject.count).to eq(4)
    end

    it 'should return each item as a hash' do
      expect(subject.first).to be_instance_of(Hash)
    end

    describe 'connection failure' do
      before do
        allow(cloudflare_access).to receive(:end_time).and_return('2017-10-9T23:55:00Z')
      end

      subject do
        VCR.use_cassette(:error) do
          cloudflare_access.logs
        end
      end

      it 'should raise an error' do
        allow(logger).to receive(:error)
        expect { subject }.to raise_error(RestClient::BadRequest)
      end

      it 'should log a descriptive error' do
        expect(logger).to receive(:error).with("{\"success\":false,\"errors\":[{\"code\":10000,\"message\":\"Authentication error\"}]}")
        expect { subject }.to raise_error(RestClient::BadRequest)
      end
    end
  end
end
